@@ -4,7 +4,4 @@ from rpyc.core.protocol import Connection
 from rpyc.core.netref import BaseNetref
 from rpyc.core.async import AsyncResult, AsyncResultTimeout
 from rpyc.core.service import Service, VoidService, SlaveService
-from rpyc.core.vinegar import GenericException, install_rpyc_excepthook
-
-install_rpyc_excepthook()
-
+from rpyc.core.vinegar import GenericException
